clear;
% Resolució equació de transferència de calor estat transitori en el
% cas unidimensional. 
%
%             du/dt-u_{xx}=f(x,t) amb x\in[0,1],
%                   u(0,t)=0      condició de contorn
%                   u(1,t)=0      condició de contorn
%                   u(x,0)=0      condició inicial

%--------------------------Dades del problema------------------------------

%{
Sabem que la solució és u(t,x)=(t+t^2)x(1-x) i fent els càlculs 
corresponents trobem que f(t,x)=(1+2t)x(1-x)+2t(t+1). Així doncs farem la
comparativa entre la solució exacta i la solució obtinguda amb FEM
%}


I=5;       
noe=I+1;    %nombre d'elements
non=noe+1;  %nombre de nodes
L=1;        %llargada del domini
h=L/noe;    %llargada de cada element
N=5;      %nombre de passos temporals
T=1;        %temps del procés transitori
dt=T/N;     %pas temporal



%--------------Construcció de la matriu de rigidesa R---------------------- 

R=zeros(I,I);   %Inicialitzem la matriu

%Cas on la llargada dels elements és una constant=h
for i=1:1:I-1
    R(i,i)=2;
    R(i,i+1)=-1;
    R(i+1,i)=-1;
end
R(I,I)=2;

R=(1/h)*R;

%display(R);


%--------------Construcció de la matriu de massa R------------------------- 

M=zeros(I,I);
%Cas on la llargada dels elements és una constant=h
for i=1:1:I-1
    M(i,i)=2*h/3;
    M(i,i+1)=h/6;
    M(i+1,i)=h/6;
end
M(I,I)=2*h/3;

%disp(M);

%---------------------Construïm la matriu de solucions---------------------
%{
Les columnes d'aquesta matriu constituiran les solucions al llarg dels 
diferents passos temporals.
%}
U=zeros(I,N+1);


%------------------------Mètode Crank Nicolson-----------------------------
%{
Hem de solucionar el sistema D*U_{n+1}=C*U_{n}+F
on D=(M+dt/2*R), C=(M-dt/2*R), F=(dt*h/2)*(f^{n+1}+f^n)
Sabem que f(t,x)=(1+2t)x(1-x)+2t(t+1)=
%}

D=(M+(dt/2)*R);
C=(M-(dt/2)*R);
F=zeros(I,1);
f_inici=zeros(I,1);
f_final=zeros(I,1);
for n=0:1:N-1
    %Construim el vector F:
    for i=1:1:I
        x=i*h;
        t=n*dt;
        f_inici(i)=(1+2*t)*x*(1-x)+2*t*(t+1);
        t=(n+1)*dt;
        f_final(i)=(1+2*t)*x*(1-x)+2*t*(t+1);
        F(i)=((dt*h)/2)*(f_final(i)+f_inici(i));
    end 
    S=(F+C*U(: ,n+1));
    U(:, n+2)=D\S;
    
    X=sprintf('Pas %d de %d',n,N);
    disp(X);
    
end

%disp(U);

%--------------Afegim al vector solució les condicions de contorn----------
U_sol=zeros(I+2,N+1);
for n=2:1:N+1
    for i=1:1:I
        U_sol(i+1,n)=U(i,n);
    end
end

%disp(U_sol);


%-------------------------Grafiquem solució exacta-------------------------
syms t x
u=@(t,x) (t+t.^2)*x.*(1-x);

figure(2);
hold on;

title('Exacta')

for a=0:dt:T
    fplot(u(a,x),[0 1]);
end

grid
legend('Exacta')
xlabel('x')
ylabel('u')

hold off



%-------------------------Grafiquem la solució-----------------------------
figure(1);
hold on;

title('FEM')
x=0:noe; 
x=x*h;        %Canvi d'escala, volem que entre x(i) i x(i+1) hi hagi un espai h
axis([0 1 0 0.5]);

plot(x',U_sol(:,1:N+1))

%{
for n=1:1:N
 plot(x',U(:,1:n))
 axis([0 1 0 f])
 drawnow();
 pause(0.1)
end
%}

grid
legend('FEM')
xlabel('x')
ylabel('u')

hold off


%--------------------Calculem l'error màxim-------------------------------
%Interpolem la funció U_sol a una funció poligonal P
for i=0:1:noe
    interv(i+1)=i*h;
end
%disp(interv);


%Calculem el màxim de la diferència entre P i U exacte, avaluant cada 0.01
errormax(1)=0;
for n=2:1:N+1
    P = griddedInterpolant(interv,U_sol(:,n));
    errormax(n)=0;
    for i=0:0.01:1
        error=abs(P(i)-u((n-1)*dt,i));
        if error>errormax(n)
            errormax(n)=error;
            errormaxrel(n)=errormax(n)/u((n-1)*dt,i);
            index(n)=i;
        end
    end
%disp('Error=');
%disp(errormax);
%disp('Error relatiu=');
%disp(errormaxrel*100);
%disp('Punt error=');
%disp(index);
end

disp(index);
disp(errormax);
disp(errormaxrel*100);

figure(3);
hold on;
title('Error')
plot(0:dt:1,errormax);
grid
xlabel('t')
ylabel(char(949))
hold off;

figure(4);
hold on;
title('Error relatiu')
plot(0:dt:1,errormaxrel);
grid
xlabel('t')
ylabel(char(949))
hold off;




